using namespace std;

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctime>
#include <iostream>
#include <fstream.h>

#include "EnergyLosses.h"


void elasticKinematics(Particle *pr, double mass, double angle)
{
  double momentum = pr->momentum()*1000.0;
  double new_momentum = momentum/(1.0 + momentum/mass*(1.0 - cos(angle)));
  pr->setMomentum(0.0, 0.0, new_momentum/1000.0);
}


int HadronCollisionalLosses(Particle *pr, const TargetMaterial *material, double length, double *outkappa, double *Stopping_Power, double *correction, double *mean_loss)
{
  // first we convert GeV -> MeV
  double momentum = pr->momentum()*1000.0;
  double mass = pr->getMass()*1000.0;
  double energy = pr->energy()*1000.0;
  double gamma = energy/mass;
  double beta = momentum/energy;
 
  // Various parameters for the Bethe-Bloch formula
  double fac = 0.1535;
  double rho = material->Density;
  double A = material->Massnumber;
  double Z = material->Chargenumber;
  double I = material->IonisationPotential;
  double smallz=pr->getCharge();

  // Density Corrections
  double eta = beta*gamma;
  double eta2 = eta*eta;
  double X = log(eta)/(2.3025851);
  double delta = 0.0;      

  if (X > material->DC_x0) {
    delta  = 2.0 * 2.3025851 * X + material->DC_C;
    if (X < material->DC_x1)
      delta += material->DC_a * pow((material->DC_x1 - X),material->DC_m);
  }

  // Shell Corrections
  double C = 0.0;
  double J = I*1000000.0;
  if (eta >= 0.1) {
    C = (0.422377 / eta2 + 0.0304043 / (eta2*eta2) - 
         0.00038106 / (eta2*eta2*eta2)) * 1E-6 * J*J +
        (3.850190 / eta2 - 0.1667989 / (eta2*eta2) +
         0.00157955 / (eta2*eta2*eta2)) * 1E-9 * J*J*J;
  }


  double Wmax = 2.0*0.511*(beta*beta*gamma*gamma) / (1.0 + 2.0*0.511/mass*sqrt(1.0 + (beta*beta*gamma*gamma)) + 0.511/mass);

  double xi = fac*Z/A/beta/beta*rho*length*smallz*smallz;  
  double kappa = xi/Wmax;
  
  double epsilon = exp(beta*beta + log((1.0 - beta*beta)*I*I/beta/beta/2.0/mass)); //TODO popravi tlele epsilon

  // This is the Hadron Bethe-Bloch formula to calculate the mean energy loss of the hadrons
  double eloss_hadron = fac*Z/A/beta/beta*smallz*smallz*rho*length*(log(2.0*0.511*beta*beta/(1.0-beta*beta)*Wmax/I/I) - 2.0*beta*beta - delta -2.0*C/Z);
  



  cout<<"------------ Hadron Bethe-Bloch formula -------------"<<endl;
  cout<<"| A: "<<A<<endl;
  cout<<"| Z: "<<Z<<endl;
  cout<<"| I: "<<I<<endl;
  cout<<"| small z: "<<smallz<<endl;
  cout<<"| mass: "<<mass<<endl;
  cout<<"| T: "<<energy - mass<<endl;
  cout<<"| Fac: "<<fac<<endl;
  cout<<"| Rho: "<<rho<<endl; 
  cout<<"| Eta: "<<eta<<endl;
  cout<<"| X0: "<<material->DC_x0<<endl;
  cout<<"| X1: "<<material->DC_x1<<endl;
  cout<<"| CC: "<<material->DC_C<<endl;
  cout<<"| a: "<<material->DC_a<<endl;
  cout<<"| mm (Not mass): "<<material->DC_m<<endl;
  cout<<"| Wmax: "<<Wmax<<endl;
  cout<<"| Kappa: "<<kappa<<endl; 
  cout<<"| Xi: "<<xi<<endl; 
  cout<<"| Epsilon: "<<epsilon<<endl; 
  cout<<"| Delta: "<<delta<<endl; 
  cout<<"| Target Length: "<<length<<endl;
  cout<<"| S (MeV cm^2 /g): "<<eloss_hadron/rho/length<<endl;
  cout<<"| Logarithmic term: "<<log(2.0*0.511*beta*beta/(1.0-beta*beta)*Wmax/I/I)<<endl;
  
  cout<<"-----------------------------------------------------"<<endl;
  
  cout<<"******************** Results: *************************"<<endl;
  cout<<"* Element: (A,Z): ("<<A<<", "<<Z<<")"<<endl;
  cout<<"* Mean Energy Loss: "<<eloss_hadron<<" MeV"<<endl;
  cout<<"*******************************************************"<<endl;


  *outkappa = kappa;
  *Stopping_Power = eloss_hadron/rho/length;
  *correction = 0.0;
  *mean_loss = eloss_hadron;


  if ( energy - mass < eloss_hadron){
    double new_energy = mass;
    double new_momentum = 0.0;
    pr->setMomentum(0.0, 0.0, new_momentum/1000.0); 
    return 1; // need to stop 
  }
  else{
    double new_energy = energy - eloss_hadron;
    double new_momentum = sqrt(new_energy*new_energy - mass*mass);
    pr->setMomentum(0.0, 0.0, new_momentum/1000.0);
    return 0;
  }

}


double IntegratedHadronCollisionalLosses(Particle *pr, const TargetMaterial *material, double length, double dx = 0.01)
{
  double outkappa;
  double Stopping_Power;
  double correction; 
  double mean_loss; 
  double mp_loss;
  double x;
  
  for (x = 0; x<=length; x+=dx){
    if (pr->momentum() <=0.0) break;
    int stop = HadronCollisionalLosses(pr, material, dx, &outkappa, &Stopping_Power, &correction,&mean_loss);
    if (stop > 0) break;
  }

  return x;

}

/*

//************************************************************
//
// Protons in Scintillator
//  
//************************************************************
void EnergyDependence_ProtonScintillator()
{

  ofstream out;
  out.open("Total_Proton_EdE_Energy_Losses2.dat");
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;


  for (double i = 100.0; i<1500.0; i = i+1.0) //Momentum in MeV/c
  {
      //double i = 349.9;
      
      Particle proton = P_proton;
      proton.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Electron Momentum in the beinning: "<<proton.momentum()<<endl;
      double mass = proton.getMass()*1000.0;

      double MomentumAtTarget = proton.momentum()*1000.0;

      //lossen during the travel
      HadronCollisionalLosses(&proton, TM_SiO2, 0.17, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Air, 50.0, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_SiO2, 0.15, &kappa, &bethe_bloch, &correction,&mean_loss); // fiberglass
      HadronCollisionalLosses(&proton, TM_Air, 240.0, &kappa, &bethe_bloch, &correction,&mean_loss);

      HadronCollisionalLosses(&proton, TM_Kapton, 0.003, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Kapton, 0.003, &kappa, &bethe_bloch, &correction,&mean_loss);

      HadronCollisionalLosses(&proton, TM_Kapton, 0.003, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&proton, TM_Kapton, 0.003, &kappa, &bethe_bloch, &correction,&mean_loss);   

      HadronCollisionalLosses(&proton, TM_Air, 84.0, &kappa, &bethe_bloch, &correction,&mean_loss);
  
      double MomentumStart = proton.momentum()*1000.0;
    
      double PathdE = IntegratedHadronCollisionalLosses(&proton, TM_VinylToluene, 0.3);   //dE-plane

      double MomentumdE = proton.momentum()*1000.0;
      double EnergyLostdE = sqrt(MomentumStart*MomentumStart + mass*mass) - sqrt(MomentumdE*MomentumdE + mass*mass);
      
      double PathE = IntegratedHadronCollisionalLosses(&proton, TM_VinylToluene, 3.0);   //E-plane
      double MomentumE = proton.momentum()*1000.0;
      double EnergyLostE = sqrt(MomentumdE*MomentumdE + mass*mass) - sqrt(MomentumE*MomentumE + mass*mass);


      cout<<"Proton Momentum at Target: "<<MomentumAtTarget<<endl;
      cout<<"Proton Momentum at Start: "<<MomentumStart<<endl;
      cout<<"Proton Momentum in the end: "<<proton.momentum()<<endl;
      cout<<"Path traveled in the dE: "<<PathdE<<endl;
      cout<<"Path traveled in the E: "<<PathE<<endl;
      cout<<"EnergyLost in in the dE: "<<EnergyLostdE<<endl;
      cout<<"EnergyLost in in the E: "<<EnergyLostE<<endl;
      out<<MomentumAtTarget<<"  "<<MomentumStart<<"  "<< PathdE<<"  "<<PathE<<"  "<<EnergyLostdE<<"  "<<EnergyLostE<<"  "<<MomentumdE<<"  "<<MomentumE<<endl; 

      

  }

   out.close();
  
}
*/
/*
//************************************************************
//
// Deuterons in Scintillator
//  
//************************************************************
void EnergyDependence_DeuteronScintillator()
{

  ofstream out;
  out.open("Total_Deuteron_EdE_Energy_Losses2.dat");
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;
  


  for (double i = 100.0; i<1500.0; i = i+1.0) //Momentum in MeV/c
  {
      //double i = 349.9;
      
      Particle deuteron = P_deuteron;
      deuteron.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Electron Momentum in the beinning: "<<deuteron.momentum()<<endl;
      double mass = deuteron.getMass()*1000.0;

      double MomentumAtTarget = deuteron.momentum()*1000.0;

      //lossen during the travel
      HadronCollisionalLosses(&deuteron, TM_SiO2, 0.17, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Air, 50.0, &kappa, &bethe_bloch, &correction,&mean_loss);
      //HadronCollisionalLosses(&deuteron, TM_SiO2, 0.0762, &kappa, &bethe_bloch, &correction,&mean_loss); // fiberglass
      HadronCollisionalLosses(&deuteron, TM_SiO2, 0.15, &kappa, &bethe_bloch, &correction,&mean_loss); // fiberglass
      HadronCollisionalLosses(&deuteron, TM_Air, 240.0, &kappa, &bethe_bloch, &correction,&mean_loss);

      HadronCollisionalLosses(&deuteron, TM_Kapton, 0.003, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Kapton, 0.003, &kappa, &bethe_bloch, &correction,&mean_loss);

      HadronCollisionalLosses(&deuteron, TM_Kapton, 0.003, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178, &kappa, &bethe_bloch, &correction,&mean_loss);
      HadronCollisionalLosses(&deuteron, TM_Kapton, 0.003, &kappa, &bethe_bloch, &correction,&mean_loss);

      HadronCollisionalLosses(&deuteron, TM_Air, 84.0, &kappa, &bethe_bloch, &correction,&mean_loss);   

  
      double MomentumStart = deuteron.momentum()*1000.0;
    
      double PathdE = IntegratedHadronCollisionalLosses(&deuteron, TM_VinylToluene, 0.3);   //dE-plane

      double MomentumdE = deuteron.momentum()*1000.0;
      double EnergyLostdE = sqrt(MomentumStart*MomentumStart + mass*mass) - sqrt(MomentumdE*MomentumdE + mass*mass);
      
      double PathE = IntegratedHadronCollisionalLosses(&deuteron, TM_VinylToluene, 3.3);   //E-plane
      double MomentumE = deuteron.momentum()*1000.0;
      double EnergyLostE = sqrt(MomentumdE*MomentumdE + mass*mass) - sqrt(MomentumE*MomentumE + mass*mass);


      cout<<"Proton Momentum at Target: "<<MomentumAtTarget<<endl;
      cout<<"Proton Momentum at Start: "<<MomentumStart<<endl;
      cout<<"Proton Momentum in the end: "<<deuteron.momentum()<<endl;
      cout<<"Path traveled in the dE: "<<PathdE<<endl;
      cout<<"Path traveled in the E: "<<PathE<<endl;
      cout<<"EnergyLost in in the dE: "<<EnergyLostdE<<endl;
      cout<<"EnergyLost in in the E: "<<EnergyLostE<<endl;
      out<<MomentumAtTarget<<"  "<<MomentumStart<<"  "<< PathdE<<"  "<<PathE<<"  "<<EnergyLostdE<<"  "<<EnergyLostE<<"  "<<MomentumdE<<"  "<<MomentumE<<endl; 

      

  }

   out.close();
  
}
*/

//************************************************************
//
// Deuterons in Scintillator
//  
//************************************************************
void EnergyDependence_DeuteronScintillator()
{

  ofstream out;
  out.open("Total_Deuteron_EdE_Energy_Losses2.dat");
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;
  


  for (double i = 200.0; i<1000.0; i = i+1.0) //Momentum in MeV/c
  {
      //double i = 349.9;
      
      Particle deuteron = P_deuteron;
      deuteron.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Electron Momentum in the beinning: "<<deuteron.momentum()<<endl;
      double mass = deuteron.getMass()*1000.0;

      double MomentumAtTarget = deuteron.momentum()*1000.0;

      //lossen during the travel
      IntegratedHadronCollisionalLosses(&deuteron, TM_SiO2, 0.17);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Air, 83.0);
      IntegratedHadronCollisionalLosses(&deuteron, TM_SiO2, 0.07812);
      //IntegratedHadronCollisionalLosses(&deuteron, TM_SiO2, 0.15624);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Air, 200.0);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Kapton, 0.003);

      double MomentumAtMWDC = deuteron.momentum()*1000.0;
   
      IntegratedHadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Kapton, 0.003);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Kapton, 0.003);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Kapton, 0.003);

      IntegratedHadronCollisionalLosses(&deuteron, TM_Air, 84.0);   
  
      double MomentumStart = deuteron.momentum()*1000.0;
    
      double PathdE = IntegratedHadronCollisionalLosses(&deuteron, TM_VinylToluene, 0.3);   //dE-plane

      double MomentumdE = deuteron.momentum()*1000.0;
      double EnergyLostdE = sqrt(MomentumStart*MomentumStart + mass*mass) - sqrt(MomentumdE*MomentumdE + mass*mass);
      
      double PathE = IntegratedHadronCollisionalLosses(&deuteron, TM_VinylToluene, 3.0*(1.0+0.0));   //E-plane
      double MomentumE = deuteron.momentum()*1000.0;
      double EnergyLostE = sqrt(MomentumdE*MomentumdE + mass*mass) - sqrt(MomentumE*MomentumE + mass*mass);


      cout<<"Proton Momentum at Target: "<<MomentumAtTarget<<endl;
      cout<<"Proton Momentum at Start: "<<MomentumStart<<endl;
      cout<<"Proton Momentum in the end: "<<deuteron.momentum()<<endl;
      cout<<"Path traveled in the dE: "<<PathdE<<endl;
      cout<<"Path traveled in the E: "<<PathE<<endl;
      cout<<"EnergyLost in in the dE: "<<EnergyLostdE<<endl;
      cout<<"EnergyLost in in the E: "<<EnergyLostE<<endl;
      out<<MomentumAtTarget<<"  "<<MomentumAtMWDC<<"  "<<MomentumAtTarget - MomentumAtMWDC<<"  "<<MomentumStart<<"  "<< PathdE<<"  "<<PathE<<"  "<<EnergyLostdE<<"  "<<EnergyLostE<<"  "<<MomentumdE<<"  "<<MomentumE<<endl; 

      

  }

   out.close();
  
}


//************************************************************
//
// Protons in Scintillator
//  
//************************************************************
void EnergyDependence_ProtonScintillator()
{

  ofstream out;
  out.open("Total_Proton_EdE_Energy_Losses2.dat");
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;


  for (double i = 200.0; i<1000.0; i = i+1.0) //Momentum in MeV/c
  {
      //double i = 349.9;
      
      Particle proton = P_proton;
      proton.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Electron Momentum in the beinning: "<<proton.momentum()<<endl;
      double mass = proton.getMass()*1000.0;

      double MomentumAtTarget = proton.momentum()*1000.0;

       //lossen during the travel
      IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.17);
      IntegratedHadronCollisionalLosses(&proton, TM_Air, 83.0);
      IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.07812);
      //IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.15624);
      IntegratedHadronCollisionalLosses(&proton, TM_Air, 200.0);
      IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);

      double MomentumAtMWDC = proton.momentum()*1000.0;

      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);
      IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);
  
      double MomentumStart = proton.momentum()*1000.0;
    
      double PathdE = IntegratedHadronCollisionalLosses(&proton, TM_VinylToluene, 0.3*(1.0+0.0));   //dE-plane

      double MomentumdE = proton.momentum()*1000.0;
      double EnergyLostdE = sqrt(MomentumStart*MomentumStart + mass*mass) - sqrt(MomentumdE*MomentumdE + mass*mass);
      
      double PathE = IntegratedHadronCollisionalLosses(&proton, TM_VinylToluene, 3.0*(1.0+0.0));   //E-plane
      double MomentumE = proton.momentum()*1000.0;
      double EnergyLostE = sqrt(MomentumdE*MomentumdE + mass*mass) - sqrt(MomentumE*MomentumE + mass*mass);


      cout<<"Proton Momentum at Target: "<<MomentumAtTarget<<endl;
      cout<<"Proton Momentum at Start: "<<MomentumStart<<endl;
      cout<<"Proton Momentum in the end: "<<proton.momentum()<<endl;
      cout<<"Path traveled in the dE: "<<PathdE<<endl;
      cout<<"Path traveled in the E: "<<PathE<<endl;
      cout<<"EnergyLost in in the dE: "<<EnergyLostdE<<endl;
      cout<<"EnergyLost in in the E: "<<EnergyLostE<<endl;
      out<<MomentumAtTarget<<"  "<<MomentumAtMWDC<<"  "<<MomentumAtTarget - MomentumAtMWDC<<"  "<<MomentumStart<<"  "<< PathdE<<"  "<<PathE<<"  "<<EnergyLostdE<<"  "<<EnergyLostE<<"  "<<MomentumdE<<"  "<<MomentumE<<endl; 

      

  }

   out.close();
  
}


//************************************************************
//
// 3He in Scintillator
//  
//************************************************************
void EnergyDependence_3HeScintillator()
{

  ofstream out;
  out.open("Total_3He_EdE_Energy_Losses2.dat");
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;


  for (double i = 100.0; i<1500.0; i = i+1.0) //Momentum in MeV/c
  {
      //double i = 349.9;
      
      Particle proton = P_He3;
      proton.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Electron Momentum in the beinning: "<<proton.momentum()<<endl;
      double mass = proton.getMass()*1000.0;

      double MomentumAtTarget = proton.momentum()*1000.0;

       //lossen during the travel
      IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.17);
      IntegratedHadronCollisionalLosses(&proton, TM_Air, 83.0);
      IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.07812);
      //IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.15624);
      IntegratedHadronCollisionalLosses(&proton, TM_Air, 200.0);

      IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);
      IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Mylar, 0.00178);
      IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);
  
      double MomentumStart = proton.momentum()*1000.0;
    
      double PathdE = IntegratedHadronCollisionalLosses(&proton, TM_VinylToluene, 0.3*(1.0+0.0));   //dE-plane

      double MomentumdE = proton.momentum()*1000.0;
      double EnergyLostdE = sqrt(MomentumStart*MomentumStart + mass*mass) - sqrt(MomentumdE*MomentumdE + mass*mass);
      
      double PathE = IntegratedHadronCollisionalLosses(&proton, TM_VinylToluene, 3.0*(1.0+0.0));   //E-plane
      double MomentumE = proton.momentum()*1000.0;
      double EnergyLostE = sqrt(MomentumdE*MomentumdE + mass*mass) - sqrt(MomentumE*MomentumE + mass*mass);


      cout<<"Proton Momentum at Target: "<<MomentumAtTarget<<endl;
      cout<<"Proton Momentum at Start: "<<MomentumStart<<endl;
      cout<<"Proton Momentum in the end: "<<proton.momentum()<<endl;
      cout<<"Path traveled in the dE: "<<PathdE<<endl;
      cout<<"Path traveled in the E: "<<PathE<<endl;
      cout<<"EnergyLost in in the dE: "<<EnergyLostdE<<endl;
      cout<<"EnergyLost in in the E: "<<EnergyLostE<<endl;
      out<<MomentumAtTarget<<"  "<<MomentumStart<<"  "<< PathdE<<"  "<<PathE<<"  "<<EnergyLostdE<<"  "<<EnergyLostE<<"  "<<MomentumdE<<"  "<<MomentumE<<endl; 

      

  }

   out.close();
  
}



//************************************************************
//
// Difference
//  
//************************************************************
void EnergyDependence_Difference(double setmom)
{

  ofstream out;
  out.open("Total_Difference_EdE_Energy_Losses3.dat");
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;
  
  Particle proton = P_proton;
  proton.setMomentum(0.0,0.0, setmom);
 
    
  IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.17);
  IntegratedHadronCollisionalLosses(&proton, TM_Air, 83.0);
  IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.07812);
  //IntegratedHadronCollisionalLosses(&proton, TM_SiO2, 0.15624);
  IntegratedHadronCollisionalLosses(&proton, TM_Air, 200.0);
  IntegratedHadronCollisionalLosses(&proton, TM_Kapton, 0.003);
  
  double MomentumProton = proton.momentum();



  for (double i = 200.0; i<700.0; i = i+1.0) //Momentum in MeV/c
  {
      //double i = 349.9;
      
      Particle deuteron = P_deuteron;
      deuteron.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Electron Momentum in the beinning: "<<deuteron.momentum()<<endl;
      double mass = deuteron.getMass()*1000.0;

      //lossen during the travel
      IntegratedHadronCollisionalLosses(&deuteron, TM_SiO2, 0.17);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Air, 83.0);
      IntegratedHadronCollisionalLosses(&deuteron, TM_SiO2, 0.07812);
      //IntegratedHadronCollisionalLosses(&deuteron, TM_SiO2, 0.15624);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Air, 200.0);
      IntegratedHadronCollisionalLosses(&deuteron, TM_Kapton, 0.003);
      
      double MomentumDeuteron = deuteron.momentum();
    
      out<<setmom<<"  "<<i/1000.0<<" | "<<MomentumProton<<"  "<<MomentumDeuteron<<"  "<<MomentumProton - MomentumDeuteron<<"  |  "<<setmom - MomentumProton<<"  "<<i/1000.0 - MomentumDeuteron<<endl; 

      

  }

   out.close();
  
}

//************************************************************
//
// Protons in Scintillator
//  
//************************************************************
void ProtonInScintillator()
{
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;


  //for (double i = 200.0; i<1000.0; i = i+1.0) //Momentum in MeV/c
  {
      double i = 350.0;
      
      Particle proton = P_proton;
      proton.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Electron Momentum in the beinning: "<<proton.momentum()<<endl;
      double mass = proton.getMass()*1000.0;

      double MomentumStart = proton.momentum()*1000.0;
    
      double Path = IntegratedHadronCollisionalLosses(&proton, TM_VinylToluene, 3.3*(1.0+0.0));   //dE-plane

      cout<<"Proton Momentum at Start: "<<MomentumStart<<endl;
      cout<<"Proton Momentum in the end: "<<proton.momentum()*1000.0<<endl;
      cout<<"Path traveled in the dE: "<<Path<<endl;

  }

  
}



int main()
{


 //EnergyDependence_LH2Target();
 //EnergyDependence_LD2Target();
 //EnergyDependence_TaTarget();
 //EnergyDependence_OpticsTarget();

 //EnergyDependence_12CTarget();

 //EnergyDependence_GeJin_e_Target();
 //EnergyDependence_GeJin_p_Target();

 //EnergyDependence_15cm_LH2Target();

 //EnergyDependence_ProtonScintillator();
 //EnergyDependence_3HeScintillator();
 //EnergyDependence_DeuteronScintillator();
 //EnergyDependence_Difference(0.47);
 //EnergyDependence_Difference(0.50);

 ProtonInScintillator();

 return 0; 
}
