using namespace std;

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctime>
#include <iostream>
#include <fstream.h>

#include "EnergyLosses.h"


void elasticKinematics(Particle *pr, double mass, double angle)
{
  double momentum = pr->momentum()*1000.0;
  double new_momentum = momentum/(1.0 + momentum/mass*(1.0 - cos(angle)));
  pr->setMomentum(0.0, 0.0, new_momentum/1000.0);
}


void HadronCollisionalLosses(Particle *pr, const TargetMaterial *material, double length, double *outkappa, double *Stopping_Power, double *correction, double *mean_loss,double *mp_loss)
{
  // first we convert GeV -> MeV
  double momentum = pr->momentum()*1000.0;
  double mass = pr->getMass()*1000.0;
  double energy = pr->energy()*1000.0;
  double gamma = energy/mass;
  double beta = momentum/energy;
 
  // Various parameters for the Bethe-Bloch formula
  double fac = 0.1535;
  double rho = material->Density;
  double A = material->Massnumber;
  double Z = material->Chargenumber;
  double I = material->IonisationPotential;
  double smallz=pr->getCharge();

  // Density Corrections
  double eta = beta*gamma;
  double eta2 = eta*eta;
  double X = log(eta)/(2.3025851);
  double delta = 0.0;      

  if (X > material->DC_x0) {
    delta  = 2.0 * 2.3025851 * X + material->DC_C;
    if (X < material->DC_x1)
      delta += material->DC_a * pow((material->DC_x1 - X),material->DC_m);
  }

  // Shell Corrections
  double C = 0.0;
  double J = I*1000000.0;
  if (eta >= 0.1) {
    C = (0.422377 / eta2 + 0.0304043 / (eta2*eta2) - 
         0.00038106 / (eta2*eta2*eta2)) * 1E-6 * J*J +
        (3.850190 / eta2 - 0.1667989 / (eta2*eta2) +
         0.00157955 / (eta2*eta2*eta2)) * 1E-9 * J*J*J;
  }


  double Wmax = 2.0*0.511*(beta*beta*gamma*gamma) / (1.0 + 2.0*0.511/mass*sqrt(1.0 + (beta*beta*gamma*gamma)) + 0.511/mass);

  double xi = fac*Z/A/beta/beta*rho*length*smallz*smallz;  
  double kappa = xi/Wmax;
  
  double epsilon = exp(beta*beta + log((1.0 - beta*beta)*I*I/beta/beta/2.0/mass)); //TODO popravi tlele epsilon

  // This is the Hadron Bethe-Bloch formula to calculate the mean energy loss of the hadrons
  double eloss_hadron = fac*Z/A/beta/beta*smallz*smallz*rho*length*(log(2.0*0.511*beta*beta/(1.0-beta*beta)*Wmax/I/I) - 2.0*beta*beta - delta -2.0*C/Z);
  //double eloss_hadron = fac*Z/A/beta/beta*smallz*smallz*rho*length*(log(2.0*0.511*beta*beta/(1.0-beta*beta)*Wmax/I/I) - 2.0*beta*beta );

  double eloss_mp = 0.0;

  ///!!!!! LOOK OUT: THIS IS AN APPROXIMATION. I don't know the correct formula for the
  /// Vavilov energy losses. Since we know, that <Eloss> - MPEloss > 0 we have chosen
  /// this criteria to decide when we use Gauss and when Landau and forget about Vavilov  
  if ((beta*beta + log(xi/Wmax) + 0.2010) < 0.0) // Landau or Vavilov
  //if (kappa <0.001)
  {
    eloss_mp =  eloss_hadron + xi*(beta*beta + log(xi/Wmax) + 0.2010);
        
  }
  else  // Gauss
  {
    eloss_mp = eloss_hadron; 
  }

  double new_energy = energy - eloss_mp;
  double new_momentum = sqrt(new_energy*new_energy - mass*mass);
  pr->setMomentum(0.0, 0.0, new_momentum/1000.0);

  cout<<"------------ Hadron Bethe-Bloch formula -------------"<<endl;
  cout<<"| A: "<<A<<endl;
  cout<<"| Z: "<<Z<<endl;
  cout<<"| I: "<<I<<endl;
  cout<<"| small z: "<<smallz<<endl;
  cout<<"| mass: "<<mass<<endl;
  cout<<"| T: "<<energy - mass<<endl;
  cout<<"| Fac: "<<fac<<endl;
  cout<<"| Rho: "<<rho<<endl; 
  cout<<"| Eta: "<<eta<<endl;
  cout<<"| X0: "<<material->DC_x0<<endl;
  cout<<"| X1: "<<material->DC_x1<<endl;
  cout<<"| CC: "<<material->DC_C<<endl;
  cout<<"| a: "<<material->DC_a<<endl;
  cout<<"| mm (Not mass): "<<material->DC_m<<endl;
  cout<<"| Wmax: "<<Wmax<<endl;
  cout<<"| Kappa: "<<kappa<<endl; 
  cout<<"| Xi: "<<xi<<endl; 
  cout<<"| Epsilon: "<<epsilon<<endl; 
  cout<<"| Delta: "<<delta<<endl; 
  cout<<"| Target Length: "<<length<<endl;
  cout<<"| S (MeV cm^2 /g): "<<eloss_hadron/rho/length<<endl;
  cout<<"| Logarithmic term: "<<log(2.0*0.511*beta*beta/(1.0-beta*beta)*Wmax/I/I)<<endl;
  
  cout<<"-----------------------------------------------------"<<endl;
  
  cout<<"******************** Results: *************************"<<endl;
  cout<<"* Element: (A,Z): ("<<A<<", "<<Z<<")"<<endl;
  cout<<"* Mean Energy Loss: "<<eloss_hadron<<" MeV"<<endl;
  cout<<"* Most Probable Energy Loss: "<<eloss_mp<<" MeV"<<endl;
  cout<<"*******************************************************"<<endl;

  *outkappa = kappa;
  *Stopping_Power = eloss_hadron/rho/length;
  *correction = eloss_mp - eloss_hadron;
  *mean_loss = eloss_hadron;
  *mp_loss = eloss_mp;


  
}


void ElectronCollisionalLosses(Particle *pr, const TargetMaterial *material, double length, double *outkappa, double *Stopping_Power, double *correction, double *mean_loss,double *mp_loss)
{
  double momentum = pr->momentum()*1000.0;
  double mass = pr->getMass()*1000.0;
  double energy = pr->energy()*1000.0;
  double gamma = energy/mass;
  double beta = momentum/energy;

  double tau = energy/0.511 - 1.0;
  double F = 1.0 - beta*beta + (tau*tau/8.0 - (2.0*tau + 1.0)*log(2.0))/((tau+1.0)*(tau+1.0));
  double fac = 0.1535;
  double rho = material->Density;
  double A = material->Massnumber;
  double Z = material->Chargenumber;
  double I = material->IonisationPotential;

  // Density Corrections
  double eta = beta*gamma;
  double eta2 = eta*eta;
  double X = log(eta)/(2.3025851);
  

  double delta = 0.0;      
  if (X > material->DC_x0) {
    delta  = 2.0 * 2.3025851 * X + material->DC_C;
    if (X < material->DC_x1)
      delta += material->DC_a * pow((material->DC_x1 - X),material->DC_m);
  }


  // Shell Corrections
  double C = 0.0;
  double J = I*1000000.0;
  if (eta >= 0.1) {
    C = (0.422377 / eta2 + 0.0304043 / (eta2*eta2) - 
         0.00038106 / (eta2*eta2*eta2)) * 1E-6 * J*J +
        (3.850190 / eta2 - 0.1667989 / (eta2*eta2) +
         0.00157955 / (eta2*eta2*eta2)) * 1E-9 * J*J*J;
  }

  double eloss = fac*Z/A/beta/beta*(log(tau*tau*(tau+2.0)/2.0/(I/0.511)/(I/0.511)) + F - delta - 2.0*C/Z)*rho*length;

  //double eloss = fac*Z/A/beta/beta*(log(tau*tau*(tau+2.0)/2.0/(I/0.511)/(I/0.511)) + F - 2.0*C/Z)*rho*length;
  
  //double eloss = fac*Z/A/beta/beta*(log(tau*tau*(tau+2.0)/2.0/(I/0.511)/(I/0.511)) + F - delta )*rho*length;
 
  //double eloss = fac*Z/A/beta/beta*(log(tau*tau*(tau+2.0)/2.0/(I/0.511)/(I/0.511)) + F )*rho*length;

  double Wmax = (energy - mass)/2.0;  // Wmax = T/2  
  double xi = fac*Z/A/beta/beta*rho*length;    
  
  double kappa = xi/Wmax;


  double eloss_mp = 0.0;

  ///!!!!! LOOK OUT THIS IS AN APPROXIMATION. I don't know the correct formula for the
  /// Vavilov energy losses. Since we know, that <Eloss> - MPEloss > 0 we have chosen
  /// this criteria to decide when we use Gauss and when Landau and forget about Vavilov  
  if (xi*(beta*beta + log(xi/Wmax) + 0.2010) < 0.0) // Landau or Vavilov
  {
    eloss_mp =  eloss + xi*(beta*beta + log(xi/Wmax) + 0.2010); //!!! Tle je plus prav
    
  }
  else  // Gauss
  {
    eloss_mp = eloss; 
  }
  
  
  double new_energy = energy - eloss_mp;
  double new_momentum = sqrt(new_energy*new_energy - mass*mass);
  pr->setMomentum(0.0, 0.0, new_momentum/1000.0);

  cout<<"------------Electron Bethe-Bloch formula -------------"<<endl;
  cout<<"| A: "<<A<<endl;
  cout<<"| Z: "<<Z<<endl;
  cout<<"| I: "<<I<<endl;
  cout<<"| Tau: "<<tau<<endl;
  cout<<"| F: "<<F<<endl;
  cout<<"| Fac: "<<fac<<endl;
  cout<<"| Rho: "<<rho<<endl; 
  cout<<"| Eta: "<<eta<<endl;
  cout<<"| X0: "<<material->DC_x0<<endl;
  cout<<"| X1: "<<material->DC_x1<<endl;
  cout<<"| CC: "<<material->DC_C<<endl;
  cout<<"| a: "<<material->DC_a<<endl;
  cout<<"| mm (Not mass): "<<material->DC_m<<endl;
  cout<<"| Wmax: "<<Wmax<<endl;
  cout<<"| Kappa: "<<kappa<<endl; 
  cout<<"| Xi: "<<xi<<endl; 
  cout<<"| Epsilon: "<<epsilon<<endl; 
  cout<<"| Delta: "<<delta<<endl; 
  cout<<"| Target Length: "<<length<<endl;
  cout<<"| S (MeV cm^2 /g): "<<eloss/rho/length<<endl;

  
  cout<<"-----------------------------------------------------"<<endl;
  
  cout<<"******************** Results: *************************"<<endl;
  cout<<"* Element: (A,Z): ("<<A<<", "<<Z<<")"<<endl;
  cout<<"* Mean Energy Loss: "<<eloss<<" MeV"<<endl;
  cout<<"* Most Probable Energy Loss: "<<eloss_mp<<" MeV"<<endl;
  cout<<"*******************************************************"<<endl;

  //*outkappa = kappa;
  *outkappa = kappa;
  *Stopping_Power = eloss/rho/length;
  *correction = eloss-eloss_mp;
  *mean_loss = eloss;
  *mp_loss = eloss_mp;
  
}



//************************************************************
//
//  Protons in the Air
//  
//************************************************************
void EnergyDependence_Protons_BigBite()
{

  ofstream out;
  out.open("Protons_BigBite_Energy_Losses.dat");
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;


  for (double i = 100.0; i<1000.0; i = i+1.0)
  {
      //double i = 360.0;
      
      Particle proton = P_proton;
      proton.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Proton Momentum in the beinning: "<<proton.momentum()<<endl;
      double mass = proton.getMass()*1000.0;
    
      HadronCollisionalLosses(&proton, TM_SiO2, 0.1, &kappa, &bethe_bloch, &correction,&mean_loss, &mp_loss);
      HadronCollisionalLosses(&proton, TM_Air, 50.0, &kappa, &bethe_bloch, &correction,&mean_loss, &mp_loss);
      HadronCollisionalLosses(&proton, TM_SiO2, 0.0762, &kappa, &bethe_bloch, &correction,&mean_loss, &mp_loss); // fiberglass
      HadronCollisionalLosses(&proton, TM_Air, 240.0, &kappa, &bethe_bloch, &correction,&mean_loss, &mp_loss);

      cout<<"Proton Momentum in the end: "<<proton.momentum()<<endl;
      out<<i<<"  "<<proton.momentum()<<"  "<< sqrt(i*i + mass*mass) - sqrt(proton.momentum()*proton.momentum()*1.0E6+ mass*mass)<<endl; 

  }

   out.close();
  
}

//************************************************************
//
//  Deuterons in the Air
//  
//************************************************************
void EnergyDependence_Deuterons_BigBite()
{

  ofstream out;
  out.open("Deuterons_BigBite_Energy_Losses.dat");
  double bethe_bloch;
  double correction;
  double kappa;
  double mean_loss;
  double mp_loss;


  for (double i = 100.0; i<1000.0; i = i+1.0)
  {
      //double i = 360.0;
      
      Particle deuteron = P_deuteron;
      deuteron.setMomentum(0.0,0.0,i*1.0/1000.0);
      cout<<"Deuteron Momentum in the beinning: "<<deuteron.momentum()<<endl;
      double mass = deuteron.getMass()*1000.0;
    
      HadronCollisionalLosses(&deuteron, TM_SiO2, 0.1, &kappa, &bethe_bloch, &correction,&mean_loss, &mp_loss);
      HadronCollisionalLosses(&deuteron, TM_Air, 50.0, &kappa, &bethe_bloch, &correction,&mean_loss, &mp_loss);
      HadronCollisionalLosses(&deuteron, TM_SiO2, 0.0762, &kappa, &bethe_bloch, &correction,&mean_loss, &mp_loss); // fiberglass
      HadronCollisionalLosses(&deuteron, TM_Air, 240.0, &kappa, &bethe_bloch, &correction,&mean_loss, &mp_loss);

      cout<<"Deuteron Momentum in the end: "<<deuteron.momentum()<<endl;
      out<<i<<"  "<<deuteron.momentum()<<"  "<< sqrt(i*i + mass*mass) - sqrt(deuteron.momentum()*deuteron.momentum()*1.0E6+ mass*mass)<<endl; 

  }

   out.close();
  
}

int main()
{

 EnergyDependence_Protons_BigBite();
 EnergyDependence_Deuterons_BigBite();
 return 0; 
}
